# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/07_value_counts.ipynb.

# %% auto 0
__all__ = ['value_counts']

# %% ../nbs/07_value_counts.ipynb 3
import advertools as adv
import pandas as pd
from plotly.subplots import make_subplots

# %% ../nbs/07_value_counts.ipynb 4
def value_counts(
    data,
    dropna=False,
    show_top=10,
    sort_others=False,
    title=None,
    style=True,
    width=900,
    height=650,
    colorscale='cividis',
):
    """
    Count the values of `data` and return a table of counts (absolute, cumulative, percentage, and cumulative percentage).

    Parameters
    ----------
    data : list, tuple, pandas.Series, pandas.DataFrame
        A collection of items to count, using any of the above data structures.
    dropna : bool
        Wether or not to drop missing values.
    show_top : int
        How many top items to show. All remaining items, will be grouped into "Others:".
    sort_others : bool
        Whether or not to put "Others" in their sorted order. The default is to have this item at the bottom.
    title : str
        The title of the chart.
    style : bool
        Whether or not to style the resulting table with a heatmap.
    width : int
        The width in pixels of the resulting figure. Set this to None to make it use the widthe of its container.
    height : int
        The width in pixels of the resulting figure. Set this to None to make it use the widthe of its container.
    colorscale : str
        Which color scale to use for the heatmaps.

    Returns
    -------
    plotly.graph_objects.Figure

    """
    if isinstance(data, (list, tuple)):
        data = pd.Series(data)
    # number of columns counted in cases it was a DataFrame:
    num_dims = 1 if len(data.shape) == 1 else data.shape[-1]
    val_counts = data.value_counts(dropna=dropna).reset_index()
    if len(val_counts) > show_top:
        others_df = pd.DataFrame(
            [['Others:'] + ['' for i in range(len(val_counts.columns)-2)] + [val_counts[show_top:]['count'].sum()]],
            columns=val_counts.columns)
        val_counts = pd.concat([
            val_counts[:show_top],
            others_df
        ])
        if sort_others:
            val_counts = val_counts.sort_values(by=['count'], ascending=False)

    count_df = (val_counts
                .assign(
                    cum_count=lambda df: df['count'].cumsum(),
                    perc=lambda df: df['count'].div(df['count'].sum()),
                    cum_perc=lambda df: df['perc'].cumsum())
                )
    if not style:
        return count_df

    count_df.insert(0, 'rank', range(1, len(count_df)+1))
    num_columns = count_df.shape[1]
    count_df = count_df.rename(columns={'cum_count': 'cum. count', 'perc': '%', 'cum_perc': 'cum. %'})
    subplot_titles = count_df.columns
    fig = make_subplots(
        cols=num_columns, rows=1,
        column_widths=[0.05] + [0.65/(num_dims) for d in range(num_dims)] + [.15, .15, .15, .15],
        horizontal_spacing=0,
        subplot_titles=subplot_titles
    )
    for i in range(count_df.shape[1]):
        tempdf = count_df.iloc[:, i].to_frame()
        fig.add_heatmap(
            z=[[100] for i in range(len(tempdf))] if i in range(num_dims+1) else tempdf,
            col=i+1,
            row=1,
            hoverinfo='skip',
            showscale=False,
            name=subplot_titles[i],
            colorscale='RdBu' if i in range(num_dims+1) else colorscale,
            texttemplate="%{text}" if i in range(num_dims+1) else "%{text:,.0f}" if subplot_titles[i].endswith('count') else "%{text:.1%}",
            textfont={'size': 16},
            text=tempdf)
    fig.layout.width = width
    fig.layout.height = height
    fig.update_xaxes(showticklabels=False, showgrid=False, zeroline=False)
    fig.update_yaxes(showticklabels=False, showgrid=False, zeroline=False, autorange='reversed')
    fig.layout.title = title
    return fig



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/08_ecdf.ipynb.

# %% auto 0
__all__ = ['ecdf']

# %% ../nbs/08_ecdf.ipynb 4
import advertools as adv
import pandas as pd
import plotly.express as px

# %% ../nbs/08_ecdf.ipynb 6
def ecdf(
    df,
    x,
    hover_name=None,
    height=None,
    width=None,
    **kwargs):
    """
    Create an empirical cumulative distribution chart, a thin wrapper around px.ecdf.

    Parameters
    ----------
    df : pandas.DataFrame
        A DataFrame from which you want to visualize one of the columns' distribution.
    x : str
        The name of the column to visualize.
    hover_name : str
        The name of the column to use for labeling the markers on mouseover.
    height : int
        The height of the chart in pixels.
    width : int
        The width of the chart in pixels.

    Returns
    -------
    ecdf : plotly.graph_objects.Figure
        A plotly chart of the desired ecdf.
    """
    df = df.sort_values(x)
    df['count below'] = range(len(df))
    df['count above'] = range(len(df)-1, -1, -1)
    df['total count'] = len(df)
    cdf = [x/len(df) for x in range(1, len(df)+1)]
    cumsum = 0
    fig = px.ecdf(
        df.dropna(subset=[x]),
        x=x,
        lines=False,
        markers=True,
        category_orders={x: df.sort_values(x)[x]},
        marginal='histogram',
        hover_name=hover_name,
        height=height,
        width=width,
        hover_data=['count below', 'count above', 'total count', hover_name],
        ecdfnorm='percent', **kwargs)
    # fig.data[0].hovertemplate = '%{customdata[3]}' + f'<b>{x.replace("_", " ").title()}</b><br><br>{x}' + ': %{x}<br>percent: %{y}<br><br>count below: %{customdata[0]:,}<br>count above: %{customdata[1]:,}<br>total count: %{customdata[2]:,}<extra></extra>'
    for data in fig.data:
        if data.type == 'histogram':
            data.opacity = 1
            continue
        if data.type in ['scatter', 'scattergl']:
            data.y = cdf[cumsum:cumsum+len(data.y)]
            cumsum += len(data.y)
            data.marker.symbol = "circle-open"
            data.marker.size = 11
            prefix = '<b>%{customdata[3]}</b>' if hover_name is not None else ''
            data.hovertemplate = prefix  + f'<br><br>{x}' + ': %{x}<br>percent: %{y:.1%}%<br><br>count below: %{customdata[0]:,}<br>count above: %{customdata[1]:,}<br>total count: %{customdata[2]:,}<extra></extra>'
    fig.update_yaxes(ticksuffix='%', showspikes=True)
    fig.update_xaxes(showspikes=True)
    if fig.data[-1].type == 'histogram':
        fig.layout.yaxis2.showspikes = False
        fig.layout.xaxis2.showspikes = False
        fig.layout.yaxis2.ticksuffix = ''
        fig.layout.yaxis2.showticklabels = True
        fig.layout.yaxis2.title = 'count'
    return fig
